@@ -7,23 +7,27 @@ import thor
 import threading
 from tornado import gen
 from tornado.options import parse_command_line
-from tornado.testing import AsyncHTTPTestCase, LogTrapTestCase
+from tornado.testing import AsyncHTTPTestCase
 from tornado.web import RequestHandler, Application, asynchronous
 import unittest
 
+
 class HelloHandler(RequestHandler):
     def get(self):
         self.write("Hello world")
 
+
 class RedirectHandler(RequestHandler):
     def get(self, path):
         self.redirect(path, status=int(self.get_argument('status', '302')))
 
+
 class PostHandler(RequestHandler):
     def post(self):
         assert self.get_argument('foo') == 'bar'
         self.redirect('/hello', status=303)
 
+
 class ChunkedHandler(RequestHandler):
     @asynchronous
     @gen.engine
@@ -34,6 +38,7 @@ class ChunkedHandler(RequestHandler):
         yield gen.Task(self.flush)
         self.finish()
 
+
 class CacheHandler(RequestHandler):
     def get(self, computed_etag):
         self.write(computed_etag)
@@ -41,6 +46,7 @@ class CacheHandler(RequestHandler):
     def compute_etag(self):
         return self._write_buffer[0]
 
+
 class TestMixin(object):
     def get_handlers(self):
         return [
@@ -49,7 +55,7 @@ class TestMixin(object):
             ('/post', PostHandler),
             ('/chunked', ChunkedHandler),
             ('/cache/(.*)', CacheHandler),
-            ]
+        ]
 
     def get_app_kwargs(self):
         return dict(static_path='.')
@@ -62,7 +68,7 @@ class TestMixin(object):
             # For our small test responses the Content-Encoding header
             # wipes out any gains from compression
             rs.CONNEG_GZIP_BAD,
-            ]
+        ]
 
     def get_allowed_errors(self):
         return []
@@ -109,10 +115,12 @@ class TestMixin(object):
     def run_redbot(self, url, method, body, headers):
         red = HttpResource(url, method=method, req_body=body,
                            req_hdrs=headers)
+
         def work():
             red.run(thor.stop)
             thor.run()
             self.io_loop.add_callback(self.stop)
+
         thread = threading.Thread(target=work)
         thread.start()
         self.wait()
@@ -229,23 +237,26 @@ class TestMixin(object):
             headers=[('If-None-Match', etags)],
             expected_status=200)
 
-class DefaultHTTPTest(AsyncHTTPTestCase, LogTrapTestCase, TestMixin):
+
+class DefaultHTTPTest(AsyncHTTPTestCase, TestMixin):
     def get_app(self):
         return Application(self.get_handlers(), **self.get_app_kwargs())
 
-class GzipHTTPTest(AsyncHTTPTestCase, LogTrapTestCase, TestMixin):
+
+class GzipHTTPTest(AsyncHTTPTestCase, TestMixin):
     def get_app(self):
         return Application(self.get_handlers(), gzip=True, **self.get_app_kwargs())
 
     def get_allowed_errors(self):
-        return super(GzipHTTPTest, self).get_allowed_errors() + [
+        return super().get_allowed_errors() + [
             # TODO: The Etag is supposed to change when Content-Encoding is
             # used.  This should be fixed, but it's difficult to do with the
             # way GZipContentEncoding fits into the pipeline, and in practice
             # it doesn't seem likely to cause any problems as long as we're
             # using the correct Vary header.
             rs.VARY_ETAG_DOESNT_CHANGE,
-            ]
+        ]
+
 
 if __name__ == '__main__':
     parse_command_line()
